@@ -1,7 +1,6 @@
 /* Modula 2 language support definitions for GDB, the GNU debugger.
 
-   Copyright (C) 1992, 1998, 2000, 2005, 2007-2012 Free Software
-   Foundation, Inc.
+   Copyright (C) 1992-2015 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -18,13 +17,16 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
-extern int m2_parse (void);	/* Defined in m2-exp.y */
+struct type_print_options;
+struct parser_state;
+
+extern int m2_parse (struct parser_state *); /* Defined in m2-exp.y */
 
 extern void m2_error (char *);	/* Defined in m2-exp.y */
 
 /* Defined in m2-typeprint.c */
 extern void m2_print_type (struct type *, const char *, struct ui_file *, int,
-			   int);
+			   int, const struct type_print_options *);
 
 extern void m2_print_typedef (struct type *, struct symbol *,
 			      struct ui_file *);
@@ -32,10 +34,10 @@ extern void m2_print_typedef (struct type *, struct symbol *,
 extern int m2_is_long_set (struct type *type);
 extern int m2_is_unbounded_array (struct type *type);
 
-extern int m2_val_print (struct type *, const gdb_byte *, int, CORE_ADDR,
-			 struct ui_file *, int,
-			 const struct value *,
-			 const struct value_print_options *);
+extern void m2_val_print (struct type *, const gdb_byte *, int, CORE_ADDR,
+			  struct ui_file *, int,
+			  const struct value *,
+			  const struct value_print_options *);
 
 extern int get_long_set_bounds (struct type *type, LONGEST *low,
 				LONGEST *high);
