@@ -1,7 +1,6 @@
 /* Specific command window processing.
 
-   Copyright (C) 1998-2001, 2004, 2007-2012 Free Software Foundation,
-   Inc.
+   Copyright (C) 1998-2015 Free Software Foundation, Inc.
 
    Contributed by Hewlett-Packard Company.
 
@@ -25,4 +24,7 @@
 
 extern unsigned int tui_dispatch_ctrl_char (unsigned int);
 
+/* Refresh the command window.  */
+extern void tui_refresh_cmd_win (void);
+
 #endif
