@@ -1,6 +1,5 @@
 /* Module support.
-   Copyright (C) 1996, 1997, 2007, 2008, 2009, 2010
-   Free Software Foundation, Inc.
+   Copyright (C) 1996-2015 Free Software Foundation, Inc.
    Contributed by Cygnus Support.
 
 This file is part of GDB, the GNU debugger.
@@ -31,6 +30,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
    TODO: Add facilities for saving/restoring state to/from a file.  */
 
+#include "gdb/remote-sim.h"
 
 /* Various function types.  */
 
